@@ -18,8 +18,7 @@ import pathos.multiprocessing as pmp
 from grammar_graph import gg
 
 import isla.derivation_tree
-from isla import language
-from isla.fuzzer import GrammarFuzzer, GrammarCoverageFuzzer
+from isla.fuzzer import GrammarCoverageFuzzer
 from isla.helpers import tree_to_string
 from isla.solver import ISLaSolver
 from isla.type_defs import Grammar, ParseTree
@@ -714,7 +713,7 @@ def evaluate_kpaths(
 
     def compute_k_paths(inp: isla.derivation_tree.DerivationTree) -> List[int]:
         try:
-            return [hash(path) for path in graph.k_paths_in_tree(inp.to_parse_tree(), k)]
+            return [hash(path) for path in graph.k_paths_in_tree(inp, k)]
         except SyntaxError as err:
             print(f"Could not compute {k}-paths, error: {err}")
             return []
